@@ -1,6 +1,9 @@
-# Version: 30-11-2012, Daniel Fischer
+# Changes:
+# 03-07-2013: Added the keepPM option
+# 19-09-2013: Changed some typos
+#             Adjusted the output
 
-jt.star.gmw <- function(X,g,goi,type,nper,alternative,mc,PARAMETERS,output){
+jt.star.gmw <- function(X,g,goi,type,nper,alternative,mc,PARAMETERS,output, keepPM=keepPM){
 
  res <- list()
  diffTests <- t(as.matrix(sort(goi)))
@@ -38,7 +41,8 @@ jt.star.gmw <- function(X,g,goi,type,nper,alternative,mc,PARAMETERS,output){
 	      class(resTemp)<-"htest"
 	      
               res[[testRun]] <- resTemp
-	      names(res)[testRun] <- paste(diffTests[testRun,],collapse="")
+	      #names(res)[testRun] <- paste(diffTests[testRun,],collapse="")
+              names(res)[testRun] <- "H1: sum(P_tt') != #t/2 , with t<t'"
 	    }
 	    if(output=="min")
 	    {
@@ -56,45 +60,18 @@ jt.star.gmw <- function(X,g,goi,type,nper,alternative,mc,PARAMETERS,output){
 #----------------------------------------------------------------------------------------------------------------------------------------
 # Case: asymptotic, two sided, X is vector
 	    res <- c()
-            stop("We do not have a two-sided version for the triple test, sorry!!!A,2S,V")
+            stop("We do not have a two-sided version for the jt* test, sorry!")
 
 	  } else  if(type=="external"){
 #----------------------------------------------------------------------------------------------------------------------------------------
-# Case: JT from the clinfun package, two.sided, X is vector
-# 	    for(testRun in 1:nrow(diffTests))
-# 	    { # Our greater and base greater are different interpretations, remeber that!!!
-# 	      testResult <- jonckheere.test(X[is.element(g,diffTests[testRun,])],g[is.element(g,diffTests[testRun,])] ,alternative="two.sided")
-# 	      PVAL <- testResult$p.value
-# 	
-# 	      names(PVAL) <- "p.value"
-# 	      STATISTIC <- testResult$statistic
-# 	      names(STATISTIC) <- "obs.value"
-# 	      ALTERNATIVE <- "two.sided"
-# 	      resTemp<-c(list(method=METHOD,data.name=DNAME,alternative=ALTERNATIVE,statistic=STATISTIC,test=TEST,p.value=PVAL,type=TYPE))
-# 	      class(resTemp)<-"htest"
-# 	      
-#               res[[testRun]] <- resTemp
-# 	      names(res)[testRun] <- paste(diffTests[testRun,],collapse="")
-# 	    }
-# 	    if(output=="min")
-# 	    {
-# 	      resMin <- matrix(NA,ncol=1,nrow=length(res))
-# 	      colnames(resMin) <- "pValues"
-# 	      rownames(resMin) <- names(res)
-# 	      for(i in 1:length(res))
-# 	      {
-# 		resMin[i,1] <- res[[i]]$p.value
-# 	      }
-# 	      res <- resMin
-# 	    }
 	    res <- c()
-	    stop("We do not have this kind of type for the tjt* test!")
+	    stop("We do not have an external version of the jt* test, sorry!")
 
           } else {
 #----------------------------------------------------------------------------------------------------------------------------------------
 # Case: other options, two sided, X is vector
 	    res <- c()
-	    stop("We do not have this kind of type for the triple test!,O,2S,V")
+	    stop("We do not have this kind of type for the jt* test, sorry!")
 	  }
 ##---------------------------------------------------------------------------------------------------------------------------------------
        } else if(alternative=="greater"){
@@ -116,7 +93,8 @@ jt.star.gmw <- function(X,g,goi,type,nper,alternative,mc,PARAMETERS,output){
 	      class(resTemp)<-"htest"
 	      
               res[[testRun]] <- resTemp
-	      names(res)[testRun] <- paste(diffTests[testRun,],collapse="")
+	      #names(res)[testRun] <- paste(diffTests[testRun,],collapse="")
+              names(res)[testRun] <- "H1: sum(P_tt') > #t/2 , with t<t'"
 	    }
 	    if(output=="min")
 	    {
@@ -134,44 +112,17 @@ jt.star.gmw <- function(X,g,goi,type,nper,alternative,mc,PARAMETERS,output){
 #----------------------------------------------------------------------------------------------------------------------------------------
 # Case: asymptotic, greater, X is vector
 	    res <- c()
-            stop("We do not have a asymptotic version for the Jonckheere-Terpstra test, sorry!!!")
+            stop("We do not have an asymptotic version for the jt* test, sorry!")
           
 	  } else  if(type=="external"){
 #----------------------------------------------------------------------------------------------------------------------------------------
-# Case: JT from the clinfun package, greater, X is vector
-# 	    for(testRun in 1:nrow(diffTests))
-# 	    { # Our greater and base greater are different interpretations, remeber that!!!
-# 	      testResult <- jonckheere.test(X[is.element(g,diffTests[testRun,])],g[is.element(g,diffTests[testRun,])] ,alternative="increasing")
-# 	      PVAL <- testResult$p.value
-# 	
-# 	      names(PVAL) <- "p.value"
-# 	      STATISTIC <- testResult$statistic
-# 	      names(STATISTIC) <- "obs.value"
-# 	      ALTERNATIVE <- "increasing"
-# 	      resTemp<-c(list(method=METHOD,data.name=DNAME,alternative=ALTERNATIVE,statistic=STATISTIC,test=TEST,p.value=PVAL,type=TYPE))
-# 	      class(resTemp)<-"htest"
-# 	      
-#               res[[testRun]] <- resTemp
-# 	      names(res)[testRun] <- paste(diffTests[testRun,],collapse="")
-# 	    }
-# 	    if(output=="min")
-# 	    {
-# 	      resMin <- matrix(NA,ncol=1,nrow=length(res))
-# 	      colnames(resMin) <- "pValues"
-# 	      rownames(resMin) <- names(res)
-# 	      for(i in 1:length(res))
-# 	      {
-# 		resMin[i,1] <- res[[i]]$p.value
-# 	      }
-# 	      res <- resMin
-# 	    }
 	    res <- c()
-	    stop("We do not have this kind of type for the jt* test!")
+	    stop("We do not have external type for the jt* test, sorry!")
 	  } else {
 #----------------------------------------------------------------------------------------------------------------------------------------
 # Case: other options, greater, X is vector
 	    res <- c()
-	    stop("We do not have this kind of type for the triple test!,O,G,V")
+	    stop("We do not have this kind of type for the triple test, sorry!")
 	  }
        } else if(alternative=="smaller"){
 ##---------------------------------------------------------------------------------------------------------------------------------------
@@ -193,7 +144,8 @@ jt.star.gmw <- function(X,g,goi,type,nper,alternative,mc,PARAMETERS,output){
 	      class(resTemp)<-"htest"
 	      
               res[[testRun]] <- resTemp
-	      names(res)[testRun] <- paste(diffTests[testRun,],collapse="")
+	      #names(res)[testRun] <- paste(diffTests[testRun,],collapse="")
+              names(res)[testRun] <- "H1: sum(P_tt') < #t/2 , with t<t'"
 	    }
 	    if(output=="min")
 	    {
@@ -211,48 +163,21 @@ jt.star.gmw <- function(X,g,goi,type,nper,alternative,mc,PARAMETERS,output){
 #----------------------------------------------------------------------------------------------------------------------------------------
 # Case: asymptotic, smaller, X is vector
 	    res <- c()
-            stop("We do not have a two-sided version for the triple test, sorry!!!,A,S,V")
+            stop("We do not have an asymptotic version for the jt* test, sorry!")
 	  
 	  } else  if(type=="external"){
 #----------------------------------------------------------------------------------------------------------------------------------------
-# Case: JT from the clinfun package, smaller, X is vector
-# 	    for(testRun in 1:nrow(diffTests))
-# 	    { # Our greater and base greater are different interpretations, remeber that!!!
-# 	      testResult <- jonckheere.test(X[is.element(g,diffTests[testRun,])],g[is.element(g,diffTests[testRun,])] ,alternative="decreasing")
-# 	      PVAL <- testResult$p.value
-# 	
-# 	      names(PVAL) <- "p.value"
-# 	      STATISTIC <- testResult$statistic
-# 	      names(STATISTIC) <- "obs.value"
-# 	      ALTERNATIVE <- "increasing"
-# 	      resTemp<-c(list(method=METHOD,data.name=DNAME,alternative=ALTERNATIVE,statistic=STATISTIC,test=TEST,p.value=PVAL,type=TYPE))
-# 	      class(resTemp)<-"htest"
-# 	      
-#               res[[testRun]] <- resTemp
-# 	      names(res)[testRun] <- paste(diffTests[testRun,],collapse="")
-# 	    }
-# 	    if(output=="min")
-# 	    {
-# 	      resMin <- matrix(NA,ncol=1,nrow=length(res))
-# 	      colnames(resMin) <- "pValues"
-# 	      rownames(resMin) <- names(res)
-# 	      for(i in 1:length(res))
-# 	      {
-# 		resMin[i,1] <- res[[i]]$p.value
-# 	      }
-# 	      res <- resMin
-# 	    }
 	    res <- c()
-	    stop("We do not have this kind of type for the jt* test!")
+	    stop("We do not have an external type for the jt* test, sorry!")
           } else {
 #----------------------------------------------------------------------------------------------------------------------------------------
 # Case: other options, one sided, X is vector
 	    res <- c()
-	    stop("We do not have this kind of type for the jt* test!")
+	    stop("We do not have this kind of type for the jt* test, sorry!")
 	  }
        } else {
 	    res <- c()
-	    stop("There is no other option than small, greater or two-sided...All other")
+	    stop("There is no other option than small, greater or two-sided...")
        }
 ## Case: X is a matrix
     } else{
@@ -277,14 +202,44 @@ jt.star.gmw <- function(X,g,goi,type,nper,alternative,mc,PARAMETERS,output){
 	     return(list(pValue=pValue,obsValue=obsValue))
             }
 
+	   innerLoopPM <- function(i,testRun){
+             nullDist <- jtStarPTest(X[is.element(g,diffTests[testRun,]),i],g[is.element(g,diffTests[testRun,])],nper) 
+             obsValue <- jtStar(X[is.element(g,diffTests[testRun,]),i],g[is.element(g,diffTests[testRun,])])
+             pValue <- 2*min(sum(nullDist<obsValue)/nper,sum(nullDist>=obsValue)/nper)
+	     return(list(pValue=pValue,obsValue=obsValue, nullDist=nullDist))
+            }
+
+	    if(keepPM){
+	        nullDistRES <- list()
+		STATISTIC <- list()
+		for(i in 1:nrow(diffTests)){
+		  nullDistRES[[i]] <- matrix(0, ncol=dimX[2],nrow=nper)
+		  STATISTIC[[i]] <- c(rep(-1,dimX[2]))
+		}
+	    }
+
 	    for(testRun in 1:nrow(diffTests))
 	    { 
 	      resTemp <- list()
-	      resInner <-  unlist(mclapply(c(1:dimX[2]),innerLoop,testRun=testRun,mc.cores=mc))
+
+	      if(keepPM==TRUE){
+   	        resInner <-  unlist(mclapply(c(1:dimX[2]),innerLoopPM,testRun=testRun,mc.cores=mc))
+		#nullDistRES <- matrix(0, ncol=dimX[2],nrow=nper)
+              } else {
+   	        resInner <-  unlist(mclapply(c(1:dimX[2]),innerLoop,testRun=testRun,mc.cores=mc))
+              }
+
 	      for(i in 1:dimX[2])
 	      {
-		PVAL <- resInner[2*i-1]
-		STATISTIC <- resInner[2*i]
+		if(keepPM==TRUE){
+                  PVAL <- resInner[nper*(i-1) + 2*(i) - 1]
+                  STATISTIC[[testRun]][i] <- resInner[nper*(i-1) + 2*i]
+                  nullDistRES[[testRun]][,i] <- resInner[(nper*(i-1) + 2*i + 1):(nper*i + 2*i)]
+                } else {
+		  PVAL <- resInner[2*i-1]
+                  STATISTIC <- resInner[2*i]
+		}
+		obsValue <- STATISTIC
 		names(PVAL) <- "p.value"
 		ALTERNATIVE <- "smaller"
 		names(STATISTIC) <- "obs.value"
@@ -292,7 +247,8 @@ jt.star.gmw <- function(X,g,goi,type,nper,alternative,mc,PARAMETERS,output){
 		class(resTemp[[i]])<-"htest"	    
 	      }
 	     res[[testRun]] <- resTemp
-	     names(res)[testRun] <- paste(diffTests[testRun,],collapse="")
+	     #names(res)[testRun] <- paste(diffTests[testRun,],collapse="")
+             names(res)[testRun] <- "H1: sum(P_tt') != #t/2 , with t<t'"
 	    }
 	    if(output=="min")
 	    {
@@ -313,57 +269,18 @@ jt.star.gmw <- function(X,g,goi,type,nper,alternative,mc,PARAMETERS,output){
 #----------------------------------------------------------------------------------------------------------------------------------------
 # Case: asymptotic, two sided, X is matrix
 	    res <- c()
-	    warning("We do not have a two-sided version for the triple test, sorry!!!,A,T,M")
+	    warning("We do not have an asymptotic test for the jt* test, sorry!")
 
 	} else if(type=="external"){
 #----------------------------------------------------------------------------------------------------------------------------------------
 # Case: base, two sided, X is matrix
-# 	   innerLoop <- function(i,testRun){
-# 	     testResult <- jonckheere.test(X[is.element(g,diffTests[testRun,]),i],g[is.element(g,diffTests[testRun,])] ,alternative="two.sided")
-#              obsValue <- testResult$statistic
-#              pValue <- testResult$p.value
-# 	     return(list(pValue=pValue,obsValue=obsValue))
-#             }
-# 
-# 	    for(testRun in 1:nrow(diffTests))
-# 	    { 
-# 	      resTemp <- list()
-# 	      resInner <-  unlist(mclapply(c(1:dimX[2]),innerLoop,testRun=testRun,mc.cores=mc))
-# 	      for(i in 1:dimX[2])
-# 	      {
-# 		PVAL <- resInner[2*i-1]
-# 		STATISTIC <- resInner[2*i]
-# 		names(PVAL) <- "p.value"
-# 		ALTERNATIVE <- "two.sided"
-# 		#DNAME <- paste("Data:",deparse(substitute(X)),", Groups:",deparse(substitute(g)),", Order: max(P",diffTests[testRun,1],diffTests[testRun,3],",P",diffTests[testRun,2],diffTests[testRun,3],")",sep="")
-# 		names(STATISTIC) <- "obs.value"
-# 		resTemp[[i]]<-c(list(method=METHOD,data.name=DNAME,alternative=ALTERNATIVE,statistic=STATISTIC,test=TEST,p.value=PVAL,type=TYPE))
-# 		class(resTemp[[i]])<-"htest"	    
-# 	      }
-# 	     res[[testRun]] <- resTemp
-# 	     names(res)[testRun] <- paste(diffTests[testRun,],collapse="")
-# 	    }
-# 	    if(output=="min")
-# 	    {
-# 	      resMin <- matrix(NA,ncol=dimX[2],nrow=length(res))
-# 	      colnames(resMin) <- colnames(X)
-# 	      rownames(resMin) <- names(res)
-# 	      for(i in 1:length(res))
-# 	      {
-# 		for(j in 1:dimX[2])
-# 		{
-# 		  resMin[i,j] <- res[[i]][[j]]$p.value
-# 		}
-# 	      }
-# 	      res <- resMin
-# 	    }
 	    res <- c()
-	    stop("We do not have this kind of type for the jt* test!")
+	    stop("We do not have an external type for the jt* test, sorry!")
           } else {
 #----------------------------------------------------------------------------------------------------------------------------------------
 # Case: other options, two sided, X is matrix
 	    res <- c()
-	    stop("We do not have this kind of type for the triple test!,O,T,M")
+	    stop("We do not have this kind of type for the jt* test, sorry!")
 	  }
     } else if(alternative=="greater"){
      #  res <- do.call(rbind,mclapply(c(1:dimX[2]),innerLoop,mc.cores=mc))
@@ -378,14 +295,44 @@ jt.star.gmw <- function(X,g,goi,type,nper,alternative,mc,PARAMETERS,output){
 	     return(list(pValue=pValue,obsValue=obsValue))
             }
 
+	   innerLoopPM <- function(i,testRun){
+             nullDist <- jtStarPTest(X[is.element(g,diffTests[testRun,]),i],g[is.element(g,diffTests[testRun,])],nper) 
+             obsValue <- jtStar(X[is.element(g,diffTests[testRun,]),i],g[is.element(g,diffTests[testRun,])])
+             pValue <- sum(nullDist>=obsValue)/nper
+	     return(list(pValue=pValue,obsValue=obsValue, nullDist=nullDist))
+            }
+
+	    if(keepPM){
+	        nullDistRES <- list()
+		STATISTIC <- list()
+		for(i in 1:nrow(diffTests)){
+		  nullDistRES[[i]] <- matrix(0, ncol=dimX[2],nrow=nper)
+		  STATISTIC[[i]] <- c(rep(-1,dimX[2]))
+		}
+	    }
+
 	    for(testRun in 1:nrow(diffTests))
 	    { 
 	      resTemp <- list()
-	      resInner <-  unlist(mclapply(c(1:dimX[2]),innerLoop,testRun=testRun,mc.cores=mc))
+
+	      if(keepPM==TRUE){
+   	        resInner <-  unlist(mclapply(c(1:dimX[2]),innerLoopPM,testRun=testRun,mc.cores=mc))
+		#nullDistRES <- matrix(0, ncol=dimX[2],nrow=nper)
+              } else {
+   	        resInner <-  unlist(mclapply(c(1:dimX[2]),innerLoop,testRun=testRun,mc.cores=mc))
+              }
+
 	      for(i in 1:dimX[2])
 	      {
-		PVAL <- resInner[2*i-1]
-		STATISTIC <- resInner[2*i]
+		if(keepPM==TRUE){
+                  PVAL <- resInner[nper*(i-1) + 2*(i) - 1]
+                  STATISTIC[[testRun]][i] <- resInner[nper*(i-1) + 2*i]
+                  nullDistRES[[testRun]][,i] <- resInner[(nper*(i-1) + 2*i + 1):(nper*i + 2*i)]
+                } else {
+		  PVAL <- resInner[2*i-1]
+                  STATISTIC <- resInner[2*i]
+		}
+		obsValue <- STATISTIC
 		names(PVAL) <- "p.value"
 		ALTERNATIVE <- "smaller"
 		names(STATISTIC) <- "obs.value"
@@ -393,7 +340,8 @@ jt.star.gmw <- function(X,g,goi,type,nper,alternative,mc,PARAMETERS,output){
 		class(resTemp[[i]])<-"htest"	    
 	      }
 	     res[[testRun]] <- resTemp
-	     names(res)[testRun] <- paste(diffTests[testRun,],collapse="")
+	     #names(res)[testRun] <- paste(diffTests[testRun,],collapse="")
+             names(res)[testRun] <- "H1: sum(P_tt') > #t/2 , with t<t'"
 	    }
 	    if(output=="min")
 	    {
@@ -413,61 +361,20 @@ jt.star.gmw <- function(X,g,goi,type,nper,alternative,mc,PARAMETERS,output){
 #----------------------------------------------------------------------------------------------------------------------------------------
 # Case: asymptotic, greater, X is matrix
 	    res <- c()
-            stop("We do not have a two-sided version for the triple test, sorry!!!A,2S,V")
+            stop("We do not have an asymptotic version for the jt* test, sorry!")
 
 	} else if(type=="external"){
 #----------------------------------------------------------------------------------------------------------------------------------------
-# Case: base, greater, X is matrix
-# 	   innerLoop <- function(i,testRun){
-# 	     testResult <- jonckheere.test(X[is.element(g,diffTests[testRun,]),i],g[is.element(g,diffTests[testRun,])] ,alternative="increasing")
-#              obsValue <- testResult$statistic
-#              pValue <- testResult$p.value
-# 	     return(list(pValue=pValue,obsValue=obsValue))
-#             }
-# 
-# 	    for(testRun in 1:nrow(diffTests))
-# 	    { 
-# 	      resTemp <- list()
-# 	      resInner <-  unlist(mclapply(c(1:dimX[2]),innerLoop,testRun=testRun,mc.cores=mc))
-# 	      for(i in 1:dimX[2])
-# 	      {
-# 		PVAL <- resInner[2*i-1]
-# 		STATISTIC <- resInner[2*i]
-# 		names(PVAL) <- "p.value"
-# 		ALTERNATIVE <- "increasing"
-# 		#DNAME <- paste("Data:",deparse(substitute(X)),", Groups:",deparse(substitute(g)),", Order: max(P",diffTests[testRun,1],diffTests[testRun,3],",P",diffTests[testRun,2],diffTests[testRun,3],")",sep="")
-# 		names(STATISTIC) <- "obs.value"
-# 		resTemp[[i]]<-c(list(method=METHOD,data.name=DNAME,alternative=ALTERNATIVE,statistic=STATISTIC,test=TEST,p.value=PVAL,type=TYPE))
-# 		class(resTemp[[i]])<-"htest"	    
-# 	      }
-# 	     res[[testRun]] <- resTemp
-# 	     names(res)[testRun] <- paste(diffTests[testRun,],collapse="")
-# 	    }
-# 	    if(output=="min")
-# 	    {
-# 	      resMin <- matrix(NA,ncol=dimX[2],nrow=length(res))
-# 	      colnames(resMin) <- colnames(X)
-# 	      rownames(resMin) <- names(res)
-# 	      for(i in 1:length(res))
-# 	      {
-# 		for(j in 1:dimX[2])
-# 		{
-# 		  resMin[i,j] <- res[[i]][[j]]$p.value
-# 		}
-# 	      }
-# 	      res <- resMin
-# 	    }
 	    res <- c()
-	    stop("We do not have this kind of type for the jt* test!")
+	    stop("We do not have an external type for the jt* test, sorry!")
 #----------------------------------------------------------------------------------------------------------------------------------------
 # Case: other, greater, X is matrix
 
 	  } else {
 	    res <- c()
-	    warning("We do not have this kind of type for the UIT!,O,G,M")
+	    warning("We do not have this kind of type for the jt* test, sorry!")
 	  }
     } else if(alternative=="smaller"){
-     #  res <- do.call(rbind,mclapply(c(1:dimX[2]),innerLoop,mc.cores=mc))
 	  if(type=="permutation"){
 #----------------------------------------------------------------------------------------------------------------------------------------
 # Case: permutation, smaller, X is matrix
@@ -479,14 +386,44 @@ jt.star.gmw <- function(X,g,goi,type,nper,alternative,mc,PARAMETERS,output){
 	     return(list(pValue=pValue,obsValue=obsValue))
             }
 
+	   innerLoopPM <- function(i,testRun){
+             nullDist <- jtStarPTest(X[is.element(g,diffTests[testRun,]),i],g[is.element(g,diffTests[testRun,])],nper) 
+             obsValue <- jtStar(X[is.element(g,diffTests[testRun,]),i],g[is.element(g,diffTests[testRun,])])
+             pValue <- sum(nullDist<obsValue)/nper
+	     return(list(pValue=pValue,obsValue=obsValue, nullDist=nullDist))
+            }
+
+	    if(keepPM){
+	        nullDistRES <- list()
+		STATISTIC <- list()
+		for(i in 1:nrow(diffTests)){
+		  nullDistRES[[i]] <- matrix(0, ncol=dimX[2],nrow=nper)
+		  STATISTIC[[i]] <- c(rep(-1,dimX[2]))
+		}
+	    }
+
 	    for(testRun in 1:nrow(diffTests))
 	    { 
 	      resTemp <- list()
-	      resInner <-  unlist(mclapply(c(1:dimX[2]),innerLoop,testRun=testRun,mc.cores=mc))
+
+	      if(keepPM==TRUE){
+   	        resInner <-  unlist(mclapply(c(1:dimX[2]),innerLoopPM,testRun=testRun,mc.cores=mc))
+		#nullDistRES <- matrix(0, ncol=dimX[2],nrow=nper)
+              } else {
+   	        resInner <-  unlist(mclapply(c(1:dimX[2]),innerLoop,testRun=testRun,mc.cores=mc))
+              }
+
 	      for(i in 1:dimX[2])
 	      {
-		PVAL <- resInner[2*i-1]
-		STATISTIC <- resInner[2*i]
+		if(keepPM==TRUE){
+                  PVAL <- resInner[nper*(i-1) + 2*(i) - 1]
+                  STATISTIC[[testRun]][i] <- resInner[nper*(i-1) + 2*i]
+                  nullDistRES[[testRun]][,i] <- resInner[(nper*(i-1) + 2*i + 1):(nper*i + 2*i)]
+                } else {
+		  PVAL <- resInner[2*i-1]
+                  STATISTIC <- resInner[2*i]
+		}
+		obsValue <- STATISTIC
 		names(PVAL) <- "p.value"
 		ALTERNATIVE <- "smaller"
 		names(STATISTIC) <- "obs.value"
@@ -494,7 +431,8 @@ jt.star.gmw <- function(X,g,goi,type,nper,alternative,mc,PARAMETERS,output){
 		class(resTemp[[i]])<-"htest"	    
 	      }
 	     res[[testRun]] <- resTemp
-	     names(res)[testRun] <- paste(diffTests[testRun,],collapse="")
+	     #names(res)[testRun] <- paste(diffTests[testRun,],collapse="")
+              names(res)[testRun] <- "H1: sum(P_tt') < #t/2 , with t<t'"
 	    }
 	    if(output=="min")
 	    {
@@ -515,66 +453,31 @@ jt.star.gmw <- function(X,g,goi,type,nper,alternative,mc,PARAMETERS,output){
 #----------------------------------------------------------------------------------------------------------------------------------------
 # Case: asymptotic, smaller, X is matrix
 	    res <- c()
-	    warning("We do not have this kind of type for the triple test!,A,S,M")
+	    warning("We do not have an asymptotic version for the jt* test, sorry!")
 
 	} else if(type=="external"){
 #----------------------------------------------------------------------------------------------------------------------------------------
-# Case: base, smaller, X is matrix
-# 	   innerLoop <- function(i,testRun){
-# 	     testResult <- jonckheere.test(X[is.element(g,diffTests[testRun,]),i],g[is.element(g,diffTests[testRun,])] ,alternative="decreasing")
-#              obsValue <- testResult$statistic
-#              pValue <- testResult$p.value
-# 	     return(list(pValue=pValue,obsValue=obsValue))
-#             }
-# 
-# 	    for(testRun in 1:nrow(diffTests))
-# 	    { 
-# 	      resTemp <- list()
-# 	      resInner <-  unlist(mclapply(c(1:dimX[2]),innerLoop,testRun=testRun,mc.cores=mc))
-# 	      for(i in 1:dimX[2])
-# 	      {
-# 		PVAL <- resInner[2*i-1]
-# 		STATISTIC <- resInner[2*i]
-# 		names(PVAL) <- "p.value"
-# 		ALTERNATIVE <- "decreasing"
-# 		#DNAME <- paste("Data:",deparse(substitute(X)),", Groups:",deparse(substitute(g)),", Order: max(P",diffTests[testRun,1],diffTests[testRun,3],",P",diffTests[testRun,2],diffTests[testRun,3],")",sep="")
-# 		names(STATISTIC) <- "obs.value"
-# 		resTemp[[i]]<-c(list(method=METHOD,data.name=DNAME,alternative=ALTERNATIVE,statistic=STATISTIC,test=TEST,p.value=PVAL,type=TYPE))
-# 		class(resTemp[[i]])<-"htest"	    
-# 	      }
-# 	     res[[testRun]] <- resTemp
-# 	     names(res)[testRun] <- paste(diffTests[testRun,],collapse="")
-# 	    }
-# 	    if(output=="min")
-# 	    {
-# 	      resMin <- matrix(NA,ncol=dimX[2],nrow=length(res))
-# 	      colnames(resMin) <- colnames(X)
-# 	      rownames(resMin) <- names(res)
-# 	      for(i in 1:length(res))
-# 	      {
-# 		for(j in 1:dimX[2])
-# 		{
-# 		  resMin[i,j] <- res[[i]][[j]]$p.value
-# 		}
-# 	      }
-# 	      res <- resMin
-# 	    }
 	    res <- c()
-	    stop("We do not have this kind of type for the jt* test!")
+	    stop("We do not have an external type for the jt* test, sorry!")
 #----------------------------------------------------------------------------------------------------------------------------------------
 # Case: other, smaller, X is matrix
 
           } else {
 	    res <- c()
-	    stop("We do not have this kind of type for the triple test!,O,S,M")
+	    stop("We do not have this kind of type for the jt* test!")
 	  }
 #----------------------------------------------------------------------------------------------------------------------------------------
 # Case: other, other, X is matrix
 
     } else {
 	    res <- c()
-	    stop("There are no other alternatives possible, sorry! All other....")
+	    stop("There are no other alternatives possible, sorry!")
      }
   }
+  if(type=="permutation"){
+    ifelse(keepPM,res <- list(p.values=res, nullDist=nullDistRES, obsValue=obsValue), res <- list(p.values=res))
+  } else {
+    res <- list(p.values=res)
+  }
   res
 }
\ No newline at end of file
